#include "StdAfx.h"
#include "ChildFrame.h"
#include "Resource.h"
#include "RenameDlg.h"
#include "AOTTools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CChildView::CChildView()
{
}

CChildView::~CChildView()
{
}

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs)
{
    if (!CWnd::PreCreateWindow(cs))
        return FALSE;

    cs.dwExStyle |= WS_EX_CLIENTEDGE;
    cs.style &= ~WS_BORDER;
    cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, ::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW + 1), NULL);

    return TRUE;
}

IMPLEMENT_DYNCREATE(CChildFrame, CMDIChildWndEx)

BEGIN_MESSAGE_MAP(CChildFrame, CMDIChildWndEx)
ON_COMMAND(ID_FILE_SAVE_AS, OnFileUntab)
ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
ON_COMMAND(ID_RENAMETAB, OnRenameTab)
ON_WM_SETFOCUS()
ON_WM_CREATE()
ON_WM_CLOSE()
ON_WM_SIZE()
END_MESSAGE_MAP()

CChildFrame::CChildFrame()
{
    m_childWnd = NULL;
}

CChildFrame::~CChildFrame()
{
}

void CChildFrame::OnFileUntab()
{
    OnClose();
}

void CChildFrame::OnFileClose()
{
    CWnd::FromHandle(m_childWnd)->SendMessage(WM_SYSCOMMAND, SC_CLOSE);
    OnClose();
}

BOOL CChildFrame::DestroyWindow()
{
    if (m_childWnd != NULL && IsWindow(m_childWnd))
    {
        ::SetWindowLong(m_childWnd, GWL_EXSTYLE, m_dwExtendedStyle);
        ::SetWindowLong(m_childWnd, GWL_STYLE, m_dwStyle);
        CWnd::FromHandle(m_childWnd)->SetParent(NULL);
        ::SetWindowPlacement(m_childWnd, &m_wPlacement);
        m_childWnd = NULL;
    }
    return CMDIChildWndEx::DestroyWindow();
}

int CChildFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CMDIChildWndEx::OnCreate(lpCreateStruct) == -1)
        return -1;

    if (!m_wndView.Create(NULL, NULL, AFX_WS_DEFAULT_VIEW, CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST, NULL))
    {
        TRACE0("Impossible de crer la fentre d'affichage\n");
        return -1;
    }

    return 0;
}

void CChildFrame::OnSetFocus(CWnd* pOldWnd)
{
    CMDIChildWndEx::OnSetFocus(pOldWnd);

    if (m_childWnd != NULL)
    {
        GetMDIFrame()->SetIcon(GetIconFromHWnd(m_childWnd, TRUE), TRUE);
        GetMDIFrame()->SetIcon(GetIconFromHWnd(m_childWnd, FALSE), FALSE);
    }

    m_wndView.SetFocus();
}

BOOL CChildFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if (m_wndView.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
        return TRUE;

    return CMDIChildWndEx::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CChildFrame::OnSize(UINT nType, int cx, int cy)
{
    if (m_childWnd != NULL && IsWindow(m_childWnd))
    {
        ::MoveWindow(m_childWnd, 0, 0, cx, cy, TRUE);
    }
    CMDIChildWndEx::OnSize(nType, cx, cy);
}

void CChildFrame::SetChild(HWND wnd)
{
    m_childWnd = wnd;

    HICON hIcon = GetIconFromHWnd(m_childWnd, FALSE);                   
    SetIcon(hIcon, FALSE);         // Set big icon

    m_wPlacement.length = sizeof m_wPlacement;                     
    ::GetWindowPlacement(m_childWnd, &m_wPlacement);
    m_dwStyle = GetWindowLong(m_childWnd, GWL_STYLE);
    m_dwExtendedStyle = ::GetWindowLong(m_childWnd, GWL_EXSTYLE);
    ::SetWindowLong(m_childWnd, GWL_STYLE, m_dwStyle & ~WS_OVERLAPPEDWINDOW & ~WS_SIZEBOX);
    CWnd::FromHandle(m_childWnd)->SetParent(this);
    TCHAR text[255];
    ::GetWindowText(m_childWnd, text, 255);
    SetWindowText(text);

    CRect rect;
    GetClientRect(rect);
    ScreenToClient(rect);
    ::SetWindowPos(m_childWnd, NULL, 0, 0, rect.Width(), rect.Height(), SWP_SHOWWINDOW);
    ::MoveWindow(m_childWnd, 0, 0, rect.Width(), rect.Height(), TRUE);
}

void CChildFrame::OnRenameTab()
{
    CRenameDlg dlg;
    GetWindowText(dlg.m_Rename);
    dlg.DoModal();
    SetWindowText(dlg.m_Rename);
}
