#include "StdAfx.h"
#include "Resource.h"
#include "MainFrame.h"
#include "ChildFrame.h"
#include "AOTMessages.h"
#include "AOTTools.h"
#include "RenameDlg.h"
#include "MainHelperQ.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWndEx)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWndEx)
ON_WM_CREATE()
ON_WM_NCDESTROY()
ON_COMMAND_RANGE(ID_TRACK_WINDOWS,ID_TRACK_WINDOWS+1000,OnTrackWindows)
ON_COMMAND(ID_RENAMEWINDOW,  OnRenameWindow)
ON_COMMAND(ID_EXIT, OnClose)
ON_WM_ENTERMENULOOP()
ON_WM_CLOSE()
END_MESSAGE_MAP()

CMainFrame::CMainFrame(int index)
{
    m_index = index;
    HINSTANCE hInst = AfxGetResourceHandle();
    m_hMDIMenu = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_TABTYPE));
    m_hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_TABTYPE));
}

CMainFrame::~CMainFrame()
{
    if (m_hMDIMenu != NULL)
        FreeResource(m_hMDIMenu);
    if (m_hMDIAccel != NULL)
        FreeResource(m_hMDIAccel);
}

void CMainFrame::SetName(CString name)
{
    m_name = name;
    SetTitle(m_name);
    SetWindowText(m_name);
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CMDIFrameWndEx::OnCreate(lpCreateStruct) == -1)
        return -1;

    CMFCVisualManagerWindows::m_b3DTabsXPTheme = TRUE;
    CMFCVisualManager::SetDefaultManager(RUNTIME_CLASS(CMFCVisualManagerWindows));

    CMDITabInfo mdiTabParams;
    mdiTabParams.m_style = CMFCTabCtrl::STYLE_3D_VS2005; // autres styles disponibles...
    mdiTabParams.m_bActiveTabCloseButton = TRUE;      // dfinir  FALSE pour placer le bouton de fermeture  droite de la zone d'onglet
    mdiTabParams.m_bTabIcons = TRUE;    // dfinir  TRUE pour activer les icnes de document sous les onglets MDI
    mdiTabParams.m_bAutoColor = FALSE;    // dfinir  FALSE pour dsactiver la coloration automatique des onglets MDI
    mdiTabParams.m_bDocumentMenu = TRUE; // activer le menu de document  l'extrmit droite de la zone d'onglet
    EnableMDITabbedGroups(TRUE, mdiTabParams);

    SetName(m_name);

    CDockingManager::SetDockingMode(DT_SMART);
    EnableAutoHidePanes(CBRS_ALIGN_ANY);

    m_Helper.translateMenu(GetMenu(), IDR_TAB);
    m_Helper.translateMenu(CMenu::FromHandle(m_hMDIMenu), IDR_TABTYPE);

    return TRUE;
}

BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam)
{
    CMenu* menu = (CMenu*)lParam;

    // Get the window text to insert
    TCHAR buffer[256];
    GetWindowText(hWnd, buffer, 256);

    DWORD style = GetWindowLong(hWnd, GWL_EXSTYLE);

    // Insert it into the menu
    if (_tcslen(buffer) && IsWindowVisible(hWnd) && (style & WS_EX_TOOLWINDOW) == 0)
    {
        int index = ID_TRACK_WINDOWS + menu->GetMenuItemCount();
        TCHAR tmp[256];
        wsprintf(tmp, _T("%s %d"), buffer, index);
        menu->AppendMenu(MF_STRING, index, buffer);
    }

    return TRUE;
}

void CMainFrame::OnEnterMenuLoop(BOOL bIsTrackPopupMenu)
{
    if (!bIsTrackPopupMenu)
    {
        HMENU tmp = GetMenu()->GetSubMenu(1)->m_hMenu;
        while (::GetMenuItemCount(tmp) > 0)
        {
            ::DeleteMenu(tmp, 0, MF_BYPOSITION);
        }

        FARPROC EnumProcInstance = MakeProcInstance((FARPROC)EnumWindowsProc, AfxGetInstanceHandle());
        EnumWindows((WNDENUMPROC)EnumProcInstance, (LPARAM)GetMenu()->GetSubMenu(1));
        FreeProcInstance(EnumProcInstance);

        CString windowName;
        GetWindowText(windowName);
        // Remove the window from choice list
        for (UINT i = 0; i < GetMenu()->GetSubMenu(1)->GetMenuItemCount(); i ++)
        {
            CString menuTitle;
            GetMenu()->GetSubMenu(1)->GetMenuString(i, menuTitle, MF_BYPOSITION);
            if (menuTitle == windowName)
            {
                GetMenu()->GetSubMenu(1)->RemoveMenu(i, MF_BYPOSITION);
                break;
            }
        }
    }
    CMDIFrameWndEx::OnEnterMenuLoop(bIsTrackPopupMenu);
}

void CMainFrame::OnTrackWindows(UINT nID)
{
    CString tmp;

    GetMenu()->GetSubMenu(1)->GetMenuString(nID, tmp, MF_BYCOMMAND);
    CWnd* wnd = CWnd::FindWindow(NULL, tmp);
    if (wnd != NULL)
    {
        TabWindow(wnd->m_hWnd);
    }
}

void CMainFrame::TabWindow(HWND hWnd)
{
    CWnd* wnd = CWnd::FromHandle(hWnd);
    if (wnd->GetParent() == NULL)
    {
        HICON hIcon = GetIconFromHWnd(hWnd, FALSE);                   
        SetIcon(hIcon, FALSE);         // Set big icon

        wnd->SetParent(this);
        CChildFrame* childWnd = (CChildFrame*)CreateNewChild(RUNTIME_CLASS(CChildFrame), IDR_TABTYPE, m_hMDIMenu, m_hMDIAccel);
        childWnd->SetChild(wnd->m_hWnd);
    }
}

CMDIChildWnd* CMainFrame::CreateNewChild(CRuntimeClass* pClass, UINT nResources, HMENU hMenu /* = NULL */, HACCEL hAccel /* = NULL */)
{
    ASSERT(pClass != NULL);
    CMDIChildWnd* pFrame = (CMDIChildWnd*)pClass->CreateObject();
    ASSERT_KINDOF(CMDIChildWnd, pFrame);

    // load the frame
    CCreateContext context;
    context.m_pCurrentFrame = this;

    pFrame->SetHandles(hMenu, hAccel);
    if (!pFrame->LoadFrame(nResources, WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, this, &context))
    {
        TRACE(traceAppMsg, 0, "Couldn't load frame window.\n");
        return NULL;
    }

    // redraw the frame and parent
    pFrame->InitialUpdateFrame(NULL, TRUE);

    return pFrame;
}


void CMainFrame::OnClose()
{
    CFrameWnd* child = GetActiveFrame();
    while (child != this)
    {
        child->SendMessage(WM_CLOSE);
        child = GetActiveFrame();
    }
    DestroyWindow();
    m_hWnd = NULL;
}

void CMainFrame::OnRenameWindow()
{
    CRenameDlg dlg;
    GetWindowText(dlg.m_Rename);
    dlg.DoModal();
    SetName(dlg.m_Rename);
    dlg.DestroyWindow();
}

void CMainFrame::PostNcDestroy()
{
    ::SendMessage(HWND_BROADCAST, AOT_REMOVE_TAB, m_index, 0);
    delete this;                        // delete hidden dialog
}

void CMainFrame::decIndex()
{
    m_index--;
}
