#include "stdafx.h"
#include "Resource.h"
#include "AOTMessages.h"
#include "TabManager.h"
#include "Registry.h"
#include "MainHelperQ.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

HMODULE hAOT;

extern "C"
{
    typedef BOOL (*LPSetHook)(HWND hWnd);
    typedef BOOL (*LPClearHook)(HWND hWnd);
}

CTabManager::CTabManager(UINT nIDTemplate) : CDialog(nIDTemplate)
{
    hAOT = LoadLibrary(_T("AOTHook.dll"));
    g_count = 0;
}

CTabManager::~CTabManager()
{
    clock.DestroyWindow();
    while (m_listFrame.GetSize() > 0)
    {
        CMainFrame* pMainFrame = m_listFrame.GetAt(0);
        m_listFrame.RemoveAt(0);
        if (IsWindow(pMainFrame->m_hWnd))
            pMainFrame->OnClose();
    }
}

BEGIN_MESSAGE_MAP(CTabManager, CDialog)
    ON_WM_INITMENUPOPUP()
    ON_WM_NCDESTROY()
    ON_WM_DESTROY()
    ON_REGISTERED_MESSAGE(AOT_TAB_HWND, TabWindow)
    ON_REGISTERED_MESSAGE(AOT_TAB_NAME, GetName)
    ON_REGISTERED_MESSAGE(AOT_REMOVE_TAB, RemoveTab)
    ON_REGISTERED_MESSAGE(AOT_TAB_SIZE, GetTabSize)
    ON_COMMAND(IDC_DISPLAYVISIBLE, OnDisplayVisible)
    ON_COMMAND(IDC_DISPLAYTRAYME, OnDisplayTrayMe)
    ON_COMMAND(IDC_DISPLAYMONITOR, OnDisplayMonitor)
    ON_COMMAND(IDC_DISPLAYCLOCK, OnDisplayClock)
    ON_COMMAND(IDC_DISPLAYCAPS, OnDisplayCaps)
    ON_COMMAND(IDC_DISPLAYTAB, OnDisplayTab)
    ON_COMMAND(IDC_DISPLAYRENAMEWND, OnDisplayRenameWnd)
END_MESSAGE_MAP()


// CTabManager message handlers

BOOL CTabManager::OnInitDialog()
{
    CDialog::OnInitDialog();

    if (hAOT != NULL)
    {
        LPSetHook pSetHook = (LPSetHook)GetProcAddress(hAOT, "SetHook");
        if (pSetHook != NULL)
        {
            (pSetHook)(m_hWnd);
        }
    }

    ShowWindow(SW_HIDE);            // hide window

    clock.Start();

    return TRUE;
}

void CTabManager::OnDestroy()
{
    CDialog::OnDestroy();           // call default proc
    if (hAOT != NULL)
    {
        LPClearHook pClearHook = (LPSetHook)GetProcAddress(hAOT, "ClearHook");
        if (pClearHook != NULL)
        {
            (pClearHook)(m_hWnd);
        }
        FreeLibrary(hAOT);
        hAOT = NULL;
    }
}

void CTabManager::PostNcDestroy()
{
    delete this;                        // delete hidden dialog
}

LRESULT CTabManager::TabWindow(WPARAM wParam, LPARAM lParam)
{
    CMainFrame* pFrame;
    if (lParam == -1)
    {
        pFrame = new CMainFrame(m_listFrame.GetSize());
        m_listFrame.Add(pFrame);
        pFrame->LoadFrame(IDR_TAB);
        pFrame->ShowWindow(SW_SHOW);
        CString name;
        name.Format(m_Helper.MyLoadString(IDS_LTAB1), (g_count++) + 1);
        pFrame->SetName(name); 
        pFrame->UpdateWindow();
    }
    else
    {
        if (lParam >= m_listFrame.GetSize())
            return FALSE;
        pFrame = m_listFrame.GetAt(lParam);
    }
    pFrame->TabWindow((HWND)wParam);
    return TRUE;
}

LRESULT CTabManager::GetName(WPARAM wParam, LPARAM lParam)
{
    if (wParam > 0 && wParam <= (UINT)m_listFrame.GetSize())
    {
        CMainFrame* pMainFrame = m_listFrame.GetAt(wParam - 1);
        if (IsWindow(pMainFrame->m_hWnd))
            return (LRESULT)pMainFrame->m_hWnd;
        else
            return NULL;
    }
    return FALSE;
}

LRESULT CTabManager::RemoveTab(WPARAM wParam, LPARAM lParam)
{
    if (wParam >= 0 && wParam < (UINT)m_listFrame.GetSize())
    {
        for (UINT i = m_listFrame.GetSize() - 1; i > wParam; i--)
        {
            CMainFrame* pMainFrame = m_listFrame.GetAt(i);
            pMainFrame->decIndex();
        }
        m_listFrame.RemoveAt(wParam);
    }
    return TRUE;
}

LRESULT CTabManager::GetTabSize(WPARAM wParam, LPARAM lParam)
{
    return m_listFrame.GetSize();
}

void CTabManager::OnInitMenuPopup(CMenu* pMenu, UINT wParam, BOOL isActive)
{
    CDialog::OnInitMenuPopup(pMenu, wParam, isActive); 

    if (hAOT != NULL)
    {
        CString buf;

        for (UINT j = 0; j < pMenu->GetMenuItemCount(); j++)
        {
            UINT menuID = pMenu->GetMenuItemID(j);
            if (menuID == IDS_LEXTENDEDFUNCTIONS)
            {
                CMenu popup;
                popup.LoadMenu(IDR_OPTION);
                m_Helper.translateMenu(&popup, IDR_OPTION);
                HMENU tmpHMenu = popup.Detach();
                if (pMenu->GetMenuItemCount() <= 4)
                {
                    for (int k = CMenu::FromHandle(tmpHMenu)->GetMenuItemCount() - 1; k >= 0; k--)
                    {
                        UINT menuID = CMenu::FromHandle(tmpHMenu)->GetMenuItemID(k);
                        if (menuID == 0)
                            pMenu->InsertMenu(j + 1, MF_SEPARATOR | MF_BYPOSITION, NULL);
                        else
                        {
                            CString buf;
                            CMenu::FromHandle(tmpHMenu)->GetMenuString(k, buf, MF_BYPOSITION);
                            pMenu->InsertMenu(j + 1, MF_BYPOSITION, menuID, buf);
                        }
                    }
                    pMenu->InsertMenu(j + 1, MF_SEPARATOR | MF_BYPOSITION, NULL);
                }
                else
                {
                    pMenu->InsertMenu(j + 1, MF_POPUP | MF_BYPOSITION, (UINT)tmpHMenu, m_Helper.MyLoadStringFromReg(IDS_LEXTENDEDFUNCTIONS));
                    pMenu->InsertMenu(j + 1, MF_SEPARATOR | MF_BYPOSITION, NULL);
                }
            }
        }
    }
    pMenu->RemoveMenu(IDS_LEXTENDEDFUNCTIONS, MF_BYCOMMAND);

    int iDisplayVisible = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayVisible"), FALSE);
    int iDisplayTrayMe = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayTrayMe"), FALSE);
    int iDisplayMonitor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayMonitor"), FALSE);
    int iDisplayCaps = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayCaps"), FALSE);
    int iDisplayTab = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayTab"), FALSE);
    int iDisplayRenameWnd = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayRenameWnd"), FALSE);
    int iDisplayClock = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayClock"), FALSE);
    pMenu->CheckMenuItem(IDC_DISPLAYVISIBLE, MF_BYCOMMAND | (iDisplayVisible ? MF_CHECKED : MF_UNCHECKED));
    pMenu->CheckMenuItem(IDC_DISPLAYTRAYME, MF_BYCOMMAND | (iDisplayTrayMe ? MF_CHECKED : MF_UNCHECKED));
    pMenu->CheckMenuItem(IDC_DISPLAYMONITOR, MF_BYCOMMAND | (iDisplayMonitor ? MF_CHECKED : MF_UNCHECKED));
    pMenu->CheckMenuItem(IDC_DISPLAYCAPS, MF_BYCOMMAND | (iDisplayCaps ? MF_CHECKED : MF_UNCHECKED));
    pMenu->CheckMenuItem(IDC_DISPLAYTAB, MF_BYCOMMAND | (iDisplayTab ? MF_CHECKED : MF_UNCHECKED));
    pMenu->CheckMenuItem(IDC_DISPLAYRENAMEWND, MF_BYCOMMAND | (iDisplayRenameWnd ? MF_CHECKED : MF_UNCHECKED));
    pMenu->CheckMenuItem(IDC_DISPLAYCLOCK, MF_BYCOMMAND | (iDisplayClock ? MF_CHECKED : MF_UNCHECKED));
}

void CTabManager::OnDisplayVisible()
{
    int iDisplayVisible = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayVisible"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayVisible"), (DWORD) ! iDisplayVisible);
}

void CTabManager::OnDisplayTrayMe()
{
    int iDisplayTrayMe = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayTrayMe"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayTrayMe"), (DWORD) ! iDisplayTrayMe);
}

void CTabManager::OnDisplayMonitor()
{
    int iDisplayMonitor = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayMonitor"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayMonitor"), (DWORD) ! iDisplayMonitor);
}

void CTabManager::OnDisplayClock()
{
    int iDisplayClock = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayClock"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayClock"), (DWORD) ! iDisplayClock);
    clock.ShowWindowClock(!iDisplayClock);
}

void CTabManager::OnDisplayCaps()
{
    int iDisplayCaps = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayCaps"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayCaps"), (DWORD) ! iDisplayCaps);
}

void CTabManager::OnDisplayRenameWnd()
{
    int iDisplayRenameWnd = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayRenameWnd"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayRenameWnd"), (DWORD) ! iDisplayRenameWnd);
}

void CTabManager::OnDisplayTab()
{
    int iDisplayTab = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayTab"), FALSE);
    CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\AOT"), _T("DisplayTab"), (DWORD) ! iDisplayTab);
}

