#include "stdafx.h"
#include "AOTMessages.h"
#include "AOTTools.h"
#include "MainHelperQ.h"

#pragma comment( lib, "psapi.lib" )

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Helper m_Helper;

CArray<CMonitor*, CMonitor*> m_MonitorArray;

HICON GetIconFromHWnd(HWND hWnd, BOOL bSmallIcon)
{
    HICON hIcon = (HICON)::SendMessage(hWnd, WM_GETICON, bSmallIcon ? ICON_SMALL : ICON_BIG, 0);                   
    if (!hIcon)
    {
        TCHAR szPath[255];

        DWORD dwProcessId;
        GetWindowThreadProcessId(hWnd, &dwProcessId);
        HANDLE process = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessId);
        GetModuleFileNameEx(process, NULL, szPath, 255);

        UINT uFlags = SHGFI_ICON | SHGFI_USEFILEATTRIBUTES;

        if (bSmallIcon)
            uFlags |= SHGFI_SMALLICON;
        else
            uFlags |= SHGFI_LARGEICON;

        SHFILEINFO sfi;
        SHGetFileInfo(szPath, FILE_ATTRIBUTE_NORMAL, &sfi, sizeof(SHFILEINFO), uFlags);

        hIcon = sfi.hIcon;
    }

    return hIcon;
}

// CMonitor

// constucts a monitor class not attached to any handle
CMonitor::CMonitor(HMONITOR hMonitor)
{
    MONITORINFO mi;

    mi.cbSize = sizeof(mi);
    ::GetMonitorInfo(hMonitor, &mi);
	m_rectMonitor.left = mi.rcMonitor.left;
	m_rectMonitor.top = mi.rcMonitor.top;
	m_rectMonitor.right = mi.rcMonitor.right;
	m_rectMonitor.bottom = mi.rcMonitor.bottom;

	m_rectWork.left = mi.rcWork.left;
	m_rectWork.top = mi.rcWork.top;
	m_rectWork.right = mi.rcWork.right;
	m_rectWork.bottom = mi.rcWork.bottom;
}

CMonitor::~CMonitor()
{
}

CRect CMonitor::GetMonitorRect() const
{
	return m_rectMonitor;
}

//
// these methods return true if any part of the item intersects the monitor rect
BOOL CMonitor::IsOnMonitor(const CPoint pt) const
{
    return m_rectMonitor.PtInRect(pt);
}

BOOL CMonitor::IsOnMonitor(const CWnd* pWnd) const
{

    ASSERT(::IsWindow(pWnd->GetSafeHwnd()));
    CRect wndRect;
    pWnd->GetClientRect(&wndRect);
    pWnd->ClientToScreen(&wndRect);

	CRect rect;
    return rect.IntersectRect(m_rectMonitor, wndRect);
}

BOOL CMonitor::IsOnMonitor(const CRect lprc) const
{
	CRect rect;
    return rect.IntersectRect(m_rectMonitor, lprc);
}

void CMonitor::CenterRectToMonitor(LPRECT lprc, const BOOL UseWorkAreaRect) const
{
    int w = lprc->right - lprc->left;
    int h = lprc->bottom - lprc->top;

	CRect rect = m_rectMonitor;
    if (UseWorkAreaRect)
		rect = m_rectWork;

    lprc->left = rect.left + (rect.Width() - w) / 2;
    lprc->top = rect.top + (rect.Height() - h) / 2;
    lprc->right = lprc->left + w;
    lprc->bottom = lprc->top + h;
}

void CMonitor::CenterWindowToMonitor(CWnd* const pWnd, const BOOL UseWorkAreaRect) const
{
    ASSERT(pWnd);
    ASSERT(::IsWindow(pWnd->m_hWnd));

    CRect rect;
    pWnd->GetWindowRect(&rect);
    CenterRectToMonitor(&rect, UseWorkAreaRect);
    CRect monRect;
    if (UseWorkAreaRect)
		monRect = m_rectWork;

	rect.InflateRect(&monRect);

    pWnd->SetWindowPos(NULL, 0, 0, rect.Width(), rect.Height(), SWP_NOREPOSITION | SWP_NOZORDER | SWP_NOACTIVATE);
    pWnd->SetWindowPos(NULL, rect.left, rect.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE);
}

//
// the work area does not include the start bar
CRect CMonitor::GetWorkAreaRect() const
{
	return m_rectWork;
}

void DeleteMonitors()
{
    for (int i = 0; i < m_MonitorArray.GetSize(); i++)
    {
        CMonitor* pMonitor = m_MonitorArray.GetAt(i);
        m_MonitorArray.SetAt(i, NULL);
        delete pMonitor;
    }
    m_MonitorArray.SetSize(0);
}

void RefreshMonitors()
{
    DeleteMonitors();
    m_MonitorArray.SetSize(::GetSystemMetrics(SM_CMONITORS));
    ADDMONITOR addMonitor;
    addMonitor.currentIndex = 0;
    ::EnumDisplayMonitors(NULL, NULL, AddMonitorsCallBack, (LPARAM)&addMonitor);
}

BOOL CALLBACK AddMonitorsCallBack(HMONITOR hMonitor, HDC hdcMonitor, LPRECT lprcMonitor, LPARAM dwData)
{
    LPADDMONITOR pAddMonitor = (LPADDMONITOR)dwData;

    CMonitor* pMonitor = new CMonitor(hMonitor);

    m_MonitorArray.SetAt(pAddMonitor->currentIndex, pMonitor);
    pAddMonitor->currentIndex++;
    CString tmp;
    tmp.Format(AOT_DISPLAY_MSG, pAddMonitor->currentIndex);
    pMonitor->AOT_DISP = ::RegisterWindowMessage(tmp);

    return TRUE;
}
