#include "StdAfx.h"
#include "Calendar.h"
#include "Resource.h"
#include "MainHelperQ.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CStringArray weekdays;
CStringArray monthsName;
CStringArray days;

HBITMAP Create24BPPDIBSection(HDC hDC, int iWidth, int iHeight)
{
    BITMAPINFO bmi;
    HBITMAP hbm;
    LPBYTE pBits;

    // Initialize header to 0s.
    ZeroMemory(&bmi, sizeof(bmi));

    // Fill out the fields you care about.
    bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth = iWidth;
    bmi.bmiHeader.biHeight = iHeight;
    bmi.bmiHeader.biPlanes = 1;
    bmi.bmiHeader.biBitCount = 24;
    bmi.bmiHeader.biCompression = BI_RGB;

    // Create the surface.
    hbm = CreateDIBSection(hDC, &bmi, DIB_RGB_COLORS, (void**)&pBits, NULL, 0);

    return(hbm);
}

BOOL BitmapsCompatible(LPBITMAP lpbm1, LPBITMAP lpbm2)
{
    return ((lpbm1->bmBitsPixel == lpbm2->bmBitsPixel) && 
			(lpbm1->bmPlanes == lpbm2->bmPlanes) && 
			(lpbm1->bmWidth == lpbm2->bmWidth) && 
			(lpbm1->bmHeight == lpbm2->bmHeight));
}

BOOL BlendImages(HBITMAP hbmSrc1, HBITMAP hbmSrc2, HBITMAP hbmDst, DWORD dwWeight1, COLORREF m_crBackColor, BOOL backgroundTransparency)
{
    BITMAP bmSrc1, bmSrc2, bmDst;
    RGBTRIPLE* lprgbSrc1, * lprgbSrc2, * lprgbDst;
    DWORD dwWidthBytes, dwWeight2;
    int x, y;

    // Only values between 0 and 255 are valid.
    if (dwWeight1 > 255)
        return FALSE;

    // Get weighting value for second source image.
    dwWeight2 = 255 - dwWeight1;

    // Get information about the surfaces you were passed.
    GetObject(hbmSrc1, sizeof(BITMAP), &bmSrc1);
    GetObject(hbmSrc2, sizeof(BITMAP), &bmSrc2);
    GetObject(hbmDst, sizeof(BITMAP), &bmDst);

    // Make sure you have data that meets your requirements.
    if (!BitmapsCompatible(&bmSrc1, &bmSrc2))
        return FALSE;
    if (!BitmapsCompatible(&bmSrc1, &bmDst))
        return FALSE;
    if (bmSrc1.bmBitsPixel != 24)
        return FALSE;
    if (bmSrc1.bmPlanes != 1)
        return FALSE;
    if (!bmSrc1.bmBits || !bmSrc2.bmBits || !bmDst.bmBits)
        return FALSE;

    dwWidthBytes = bmDst.bmWidthBytes;

    // Initialize the surface pointers.
    lprgbSrc1 = (RGBTRIPLE *)bmSrc1.bmBits;
    lprgbSrc2 = (RGBTRIPLE *)bmSrc2.bmBits;
    lprgbDst = (RGBTRIPLE *)bmDst.bmBits;

    for (y = 0; y < bmDst.bmHeight; y++)
    {
        for (x = 0; x < bmDst.bmWidth; x++)
        {
            if (backgroundTransparency && m_crBackColor == RGB(lprgbSrc1[x].rgbtRed, lprgbSrc1[x].rgbtGreen, lprgbSrc1[x].rgbtBlue))
            {
                lprgbDst[x].rgbtRed = lprgbSrc2[x].rgbtRed;
                lprgbDst[x].rgbtGreen = lprgbSrc2[x].rgbtGreen;
                lprgbDst[x].rgbtBlue = lprgbSrc2[x].rgbtBlue;
            }
            else
            {
                lprgbDst[x].rgbtRed = (BYTE)((((DWORD)lprgbSrc1[x].rgbtRed * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtRed * dwWeight2)) >> 8);
                lprgbDst[x].rgbtGreen = (BYTE)((((DWORD)lprgbSrc1[x].rgbtGreen * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtGreen * dwWeight2)) >> 8);
                lprgbDst[x].rgbtBlue = (BYTE)((((DWORD)lprgbSrc1[x].rgbtBlue * dwWeight1) + ((DWORD)lprgbSrc2[x].rgbtBlue * dwWeight2)) >> 8);
            }
        }

        // Move to next scan line.
        lprgbSrc1 = (RGBTRIPLE *)((LPBYTE)lprgbSrc1 + dwWidthBytes);
        lprgbSrc2 = (RGBTRIPLE *)((LPBYTE)lprgbSrc2 + dwWidthBytes);
        lprgbDst = (RGBTRIPLE *)((LPBYTE)lprgbDst + dwWidthBytes);
    }

    return TRUE;
}

BOOL DoAlphaBlend(HDC hdcDest, // Handle to destination DC.
  int nXOriginDest, // X-coord of upper-left corner.
  int nYOriginDest, // Y-coord of upper-left corner.
  int nWidthDest, // Destination width.
  int nHeightDest, // Destination height.
  HDC hdcSrc, // Handle to source DC.
  int nXOriginSrc, // X-coord of upper-left corner.
  int nYOriginSrc, // Y-coord of upper-left corner.
  int nWidthSrc, // Source width.
  int nHeightSrc, // Source height.
  DWORD dwSourceWeight, // Source weighting (between 0 and 255).
  COLORREF m_crBackColor, // Background color
  BOOL backgroundTransparency) // BackgroundTransparency
{
    // Create surfaces for sources and destination images.
    HBITMAP hbmSrc1 = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
    HBITMAP hbmSrc2 = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);
    HBITMAP hbmDst = Create24BPPDIBSection(hdcDest, nWidthDest, nHeightDest);

    // Create HDCs to hold our surfaces.
    HDC hdcSrc1 = CreateCompatibleDC(hdcDest);
    HDC hdcSrc2 = CreateCompatibleDC(hdcDest);
    HDC hdcDst = CreateCompatibleDC(hdcDest);

    // Prepare the surfaces for drawing.
    SelectObject(hdcSrc1, hbmSrc1);
    SelectObject(hdcSrc2, hbmSrc2);
    SelectObject(hdcDst, hbmDst);
    SetStretchBltMode(hdcSrc1, COLORONCOLOR);
    SetStretchBltMode(hdcSrc2, COLORONCOLOR);

    // Capture a copy of the source area.
    StretchBlt(hdcSrc1, 0, 0, nWidthDest, nHeightDest, hdcSrc, nXOriginSrc, nYOriginSrc, nWidthSrc, nHeightSrc, SRCCOPY);

    // Capture a copy of the destination area.
    StretchBlt(hdcSrc2, 0, 0, nWidthDest, nHeightDest, hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, SRCCOPY);

    // Blend the two source areas to create the destination image.
    BOOL bReturn = BlendImages(hbmSrc1, hbmSrc2, hbmDst, dwSourceWeight, m_crBackColor, backgroundTransparency);


    // Clean up objects you do not need any longer.
    // You cannot delete an object that's selected into an
    // HDC so delete the HDC first.
    DeleteDC(hdcSrc1);
    DeleteDC(hdcSrc2);
    DeleteObject(hbmSrc1);
    DeleteObject(hbmSrc2);


    // Display the blended (destination) image to the target HDC.
    if (bReturn)
    {
        BitBlt(hdcDest, nXOriginDest, nYOriginDest, nWidthDest, nHeightDest, hdcDst, 0, 0, SRCCOPY);
    }


    // Clean up the rest of the objects you created.
    DeleteDC(hdcDst);
    DeleteObject(hbmDst);

    return bReturn;
}

CCalendar::CCalendar(BOOL startOnSunday, CString szFont, int nTextSize, COLORREF crTextColor, COLORREF crBackColor, int transparency, BOOL backgroundTransparency)
{
    m_startOnSunday = startOnSunday;

    m_szFontName = _T("Garamond");
    if (!szFont.IsEmpty())
        m_szFontName = szFont;

    m_nTextSize = 24;
    if (nTextSize > 0)
        m_nTextSize = nTextSize;

    m_crTextColor = crTextColor;
    m_crBackColor = crBackColor;
	m_nTransparency = transparency;
	m_nBackgroundTransparency = backgroundTransparency;

	weekdays.SetSize(8);
	CString dayList = m_Helper.MyLoadString(IDS_LDAYLIST);
    int startPos = 0, i = 0;
    while (dayList.Find(_T(","), startPos) != -1 && i < 6)
    {
        CString tmp = dayList.Mid(startPos, dayList.Find(_T(","), startPos) - startPos);
		weekdays[i + ((m_startOnSunday) ? 1 : 0)] = tmp;
        startPos = dayList.Find(_T(","), startPos) + 1;
        i++;
    }
	weekdays[(m_startOnSunday) ? 0 : 6] = dayList.Mid(startPos);
    CString monthList = m_Helper.MyLoadString(IDS_LMONTHLIST);
    startPos = 0, i = 0;
	monthsName.SetSize(12);
    while (monthList.Find(_T(","), startPos) != -1 && i < 11)
    {
        CString tmp = monthList.Mid(startPos, monthList.Find(_T(","), startPos) - startPos);
		monthsName[i] = tmp;
        startPos = monthList.Find(_T(","), startPos) + 1;
        i++;
    }
	monthsName[11] = monthList.Mid(startPos);
}

CCalendar::~CCalendar()
{
}

void CCalendar::DrawCalendar(HDC hDC, int xPos, int yPos)
{
    // Finding the text width and height of the font to be used //
    TEXTMETRIC tmText;
    HFONT hFont, hOldFont;
    HPEN hPen, hOldPen;
    HBRUSH hBackBrush = CreateSolidBrush(m_crBackColor);
    int nXPos, nYPos;

    HDC hMemDC = ::CreateCompatibleDC(hDC);               // Memory Device context
    HBITMAP hBmp = ::CreateCompatibleBitmap(hDC, 800, 600);             // Bitmap

    ::SelectObject(hMemDC, hBmp);

    hFont = CreateFont(m_nTextSize, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, 0, 0, 0, FF_DONTCARE, m_szFontName);
    hPen = ::CreatePen(PS_SOLID, 1, m_crTextColor);

    hOldFont = (HFONT)::SelectObject(hMemDC, hFont);
    hOldPen = (HPEN)::SelectObject(hMemDC, hPen);
    ::GetTextMetrics(hMemDC, &tmText);
    ::SetTextColor(hMemDC, m_crTextColor);
    ::SetBkMode(hMemDC, TRANSPARENT);

    int charWidth = (tmText.tmMaxCharWidth + 2 * tmText.tmAveCharWidth);
    int charHeight = tmText.tmHeight + 4;

    CTime t = CTime::GetCurrentTime();
    CTime dt(t.GetYear(), t.GetMonth(), 1, 0, 0, 0);
    int nDay = dt.GetDayOfWeek() - (m_startOnSunday ? 1 : 2);
    int nMonth = t.GetMonth();
    int nYear = t.GetYear();

    if (nDay < 0)
        nDay += 7;
    while (nDay > 7)
        nDay -= 7;
    // Check for leap year

	int month[] =
	{
		31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
	};
	month[1] = ((nYear % 400) == 0 || ((nYear % 4) == 0 && (nYear % 100) != 0))? 29 : 28;

    CString day;

    days.SetSize(0);
    days.SetSize(42 * 4);

    CString tmp;
    tmp.Format(_T("%s %d"), monthsName[nMonth - 1], nYear);

    for (int i = (nDay - (m_startOnSunday ? 0 : 0)),j = 1; i < 42 && j <= month[nMonth - 1] ; i++)
    {
		day.Format(_T("%d"), j);
        days[i] = day;
        j++;
        if (i == 41)
            i = -1;
    }

    int width;               // Width of the calendar
    int height;              // Height of the calendar

	width = 15 * charWidth / 2;
    height = 8 * charHeight;
    if (days[35].GetLength() == 0)
        height -= charHeight;
    width += (width % 2);
    height += (height % 2);

    // fill back ground color
    ::FillRect(hMemDC, &CRect(0, 0, width, height), hBackBrush);

    CRect rectDay(0, 0, tmp.GetLength() * tmText.tmMaxCharWidth / 2, charHeight);

    DrawText(hMemDC, tmp, tmp.GetLength(), &rectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);

    nXPos = nYPos = 0;
    //nYPos -=(charHeight/2);

    for (int n = 0; n < 7 * 7; n++)
    {
        if (!(n % 7))
        {
            nXPos = 0;
            nYPos += charHeight;
        }

        rectDay.left = nXPos;
        rectDay.top = nYPos;
        rectDay.right = rectDay.left + charWidth;
        rectDay.bottom = rectDay.top + charHeight;

        if (n < 7)
        {
			::DrawText(hMemDC, weekdays[n], (int)_tcslen(weekdays[n]), &rectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
        }
        else
        {
            if (days[n - 7].GetLength() != 0)
            {
                if (t.GetDay() == _tstoi(days[n - 7]))
                {
                    ::FillRect(hMemDC, &rectDay, CreateSolidBrush(GetSysColor(COLOR_DESKTOP)));
                    DrawText(hMemDC, days[n - 7], (int)_tcslen(days[n - 7]), &rectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
                }
                else
                    DrawText(hMemDC, days[n - 7], (int)_tcslen(days[n - 7]), &rectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
            }
        }
        nXPos += charWidth;
    }

    ::SelectObject(hMemDC, hOldPen);
    ::SelectObject(hMemDC, hOldFont);
    ::DeleteObject(hPen);
    ::DeleteObject(hBackBrush);
    ::DeleteObject(hFont);

	DoAlphaBlend(hDC, xPos, yPos, width, height, hMemDC, 0, 0, width, height, m_nTransparency, m_crBackColor, m_nBackgroundTransparency);

	::DeleteObject(hBmp);
    ::DeleteDC(hMemDC);
}
