// ScreenSelector.cpp : fichier d'implmentation
//

#include "stdafx.h"
#include "resource.h"       // main symbols
#include "ScreenSelector.h"
#include "GdiplusWrapper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Bote de dialogue CScreenSelector

IMPLEMENT_DYNAMIC(CScreenSelector, CStatic)

CScreenSelector::CScreenSelector()
{
	m_nbScreens = 1;
	m_nbCols = 1; 
	m_nbRows = 1;
	m_curX = 1;
	m_curY = 1;

	m_border = 3;
	m_squareSize = 20;
	m_cellSpacing = 2;

	m_ref.x = -1;
	m_ref.y = -1;
}

CScreenSelector::~CScreenSelector()
{
}

BEGIN_MESSAGE_MAP(CScreenSelector, CStatic)
	ON_WM_MOUSEMOVE()
	ON_WM_SHOWWINDOW()
END_MESSAGE_MAP()

void CScreenSelector::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CStatic::OnShowWindow(bShow, nStatus);
	if(nStatus == SW_HIDE)
	{
		m_point = m_ref;
	}
	Resize();
}

void CScreenSelector::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	CDC* pDC = CDC::FromHandle(lpDrawItemStruct->hDC);
	
	CRect rect;
	GetClientRect(&rect);
	ClientToScreen(&rect);
    HDC hdc1 = CreateCompatibleDC(lpDrawItemStruct->hDC);
    HBITMAP bm1 = CreateCompatibleBitmap(lpDrawItemStruct->hDC, rect.Width(), rect.Height());
    SelectObject(hdc1, bm1);

	HBRUSH hBrushHightLight = CreateSolidBrush(::GetSysColor(COLOR_HIGHLIGHT));
	HBRUSH hBrushBack = CreateSolidBrush(RGB(255, 255, 255));

	CRect r(0, 0, rect.Width(), rect.Height());
	r.DeflateRect(1, 1);
	CDC::FromHandle(hdc1)->FillRect(rect, CBrush::FromHandle(hBrushBack));
	CDC::FromHandle(hdc1)->FillSolidRect(r, RGB(255, 255, 255));

	int x=0, y=0;
	for(x=0;x < DetSize(rect.Width(), true);x++)
	{
		for(y=0;y < DetSize(rect.Height(), true);y++)
		{
			r.left   = CalcSize(x, false);
			r.top    = CalcSize(y, false);
			r.right  = CalcSize(x + 1, false) - m_cellSpacing;
			r.bottom = CalcSize(y + 1, false) - m_cellSpacing;

			r.DeflateRect(1, 1);
			CDC::FromHandle(hdc1)->FillSolidRect(r, RGB(0, 0, 0));
			r.DeflateRect(1, 1);
			if(x > m_curX - 1 || y > m_curY - 1)
			{
				CDC::FromHandle(hdc1)->FillRect(r, CBrush::FromHandle(hBrushBack));
				if(x >= m_nbCols && y >= m_nbRows)
					CDC::FromHandle(hdc1)->FillSolidRect(r, RGB(0, 0, 0));
			}
			else
			{
				CDC::FromHandle(hdc1)->FillRect(r, CBrush::FromHandle(hBrushHightLight));
			}
		}
	}

	r.left   = CalcSize(0, false);
	r.top    = CalcSize(y - 1, false);
	r.right  = CalcSize(x, false) - m_cellSpacing;
	r.bottom = CalcSize(y, false) - m_cellSpacing;

	r.DeflateRect(1, 1);
    CPen pen(PS_SOLID, 1, RGB(0, 0, 0));
    CDC::FromHandle(hdc1)->SelectObject(&pen);

    LOGFONT lfont;
    memset(&lfont, 0, sizeof(LOGFONT));
    lfont.lfCharSet = DEFAULT_CHARSET;
	_tcscpy(lfont.lfFaceName, _T("Arial"));
    lfont.lfHeight = 78;
    CFont myFont;
    myFont.CreatePointFontIndirect(&lfont);
    CDC::FromHandle(hdc1)->SelectObject(&myFont);

	CDC::FromHandle(hdc1)->Rectangle(r);
	r.DeflateRect(2, 2);
	CString label;
	label.Format(_T("[%d, %d]"), m_curX, m_curY);
	CDC::FromHandle(hdc1)->SetBkColor(RGB(255, 255, 255));
	
	CDC::FromHandle(hdc1)->DrawText(label, &r, DT_VCENTER | DT_CENTER | DT_SINGLELINE);
	
    if (gdiplusWrapper.IsInitialized())
    {
        Graphics* myGraphics = new Graphics(pDC->m_hDC);
        Bitmap* pBmp = new Bitmap(bm1, NULL);
        myGraphics->DrawImage(pBmp, 0, 0);
        delete pBmp;
        delete myGraphics;
    }
    else
        pDC->StretchBlt(rect.left, rect.top, rect.Width(), rect.Height(), CDC::FromHandle(hdc1), 0, 0, rect.Width(), rect.Height(), SRCCOPY);

    DeleteDC(hdc1);
    DeleteObject(bm1);
}

int CScreenSelector::CalcSize(int x, bool includeBorder)
{
	int res = (m_squareSize + m_cellSpacing) * x + m_border + (includeBorder? (m_border - m_cellSpacing) : 0);
	return res;
}

int CScreenSelector::DetSize(int x, bool includeBorder)
{
	int res = (x - m_border - (includeBorder? (m_border - m_cellSpacing - 1) : 0)) / (m_squareSize + m_cellSpacing);
	return res;
}

void CScreenSelector::Resize()
{
	int maxX = m_curX + 1;
	int maxY = m_curY + 1;

	if(m_curX > m_nbCols)
	{
		if(maxX > m_nbCols * m_nbScreens)
			maxX = m_nbCols * m_nbScreens;
		maxY = m_nbRows;
	}
	if(m_curY > m_nbRows)
	{
		maxX = m_nbCols;
		if(maxY > m_nbRows * m_nbScreens)
			maxY = m_nbRows * m_nbScreens;
	}

	CRect rect, rectParent;
	GetWindowRect(&rect);
	GetParent()->ScreenToClient(&rect);
	GetParent()->GetClientRect(&rectParent);

	if(m_ref.x == -1)
	{
		m_ref = rect.TopLeft();
		m_point = m_ref;
	}

	int sizeX = CalcSize(maxX, true);
	int sizeY = CalcSize(maxY + 1, true);

	while(m_point.x + sizeX > rectParent.right)
	{
		m_point.x -= CalcSize(2, true);
	}

	while(m_point.y + sizeY > rectParent.bottom)
	{
		m_point.y -= CalcSize(2, true);
	}

	SetWindowPos(NULL, m_point.x, m_point.y, sizeX, sizeY, SWP_NOZORDER);
	Invalidate();
}
void CScreenSelector::OnMouseMove(UINT nFlags, CPoint point)
{
	m_curX = DetSize(point.x, false) + 1;
	m_curY = DetSize(point.y, false) + 1;

	if(m_curX >= m_nbCols * m_nbScreens)
		m_curX = m_nbCols * m_nbScreens;
	if(m_curY >= m_nbRows * m_nbScreens)
		m_curY = m_nbRows * m_nbScreens;

	Resize();
}

void CScreenSelector::SetNbScreens(int nbScreens)
{
	m_nbScreens = nbScreens;
}

void CScreenSelector::SetNbRows(int nbRows)
{
	m_nbRows = nbRows;
}


void CScreenSelector::SetNbCols(int nbCols)
{
	m_nbCols = nbCols;
}


void CScreenSelector::SetCurX(int curX)
{
	m_curX = curX;
}


void CScreenSelector::SetCurY(int curY)
{
	m_curY = curY;
}

int CScreenSelector::GetCurX()
{
	return m_curX;
}

int CScreenSelector::GetCurY()
{
	return m_curY;
}
